* @20i@
* Function: setout
* Description:
* Linearly interpolate the cumulative optical depth (calculated from 
* @code{dtauc} to get approximate @code{utau} corresponding to altitudes 
* @code{zout}.
*
* Parameters:
*    REAL(*) dtauc:  Optical depth of each layer. (input)
*    INTEGER nlyr:   Number of atmospheric layers. (input).
*    INTEGER ntau:   Number of output altitudes. (input).
*    REAL(*) zd:     The altitude of each level in km. @code{zd(nlyr)} is the 
*                    bottom of the atmosphere. nlyr+1 elements. (input).
*    REAL(*) zout:   The altitude of @code{ntau} output altitudes. (input).
*    REAL(*) utau:   The optical depth at @code{ntau} altitudes. (output).
*
* Return value:
* Example:
* See @file{uvspec.c}.
* Files: 
* @file{param.inc} with parameter @code{maxcly} must be present in the 
* same directory as @file{wcloud.f}.
* Known bugs:
* Author:  Arve Kylling    
*     
* @i20@
*
      SUBROUTINE setout( dtauc, nlyr, ntau, utau, zd, zout)
*
      IMPLICIT LOGICAL (A-Z)    ! To nearly get strong typing
      INCLUDE 'param.inc'
*
      INTEGER itype, lc, lu, nlyr, ntau
      LOGICAL usrtau
      REAL dtauc(*), hh, utau(*), zd(0:*), zout(*), utauout
      REAL dummy(maxcly+1)
*
      dummy(1) = 0.
      DO  lc = 1, nlyr
         dummy(lc+1) = dummy(lc) + dtauc(lc)
      ENDDO
*
      itype = 2
      DO lu = 1, ntau
         CALL inter( maxulv, nlyr+1, itype, zout(lu),
     $        zd(0), dummy, utauout, hh)  
         utau(ntau-lu+1) = utauout
      ENDDO
*
      RETURN
      END
