#
# Copyright (C) 1997 Arve Kylling
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 1, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY of FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To obtain a copy of the GNU General Public License write to the
# Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
# USA.

#-------------------------------------------------------------------------

#####
# The documentation going into the libRadtran documentation is found in 
# the file read_o3_tab.c. This so because sdoc do not read perl yet.
#####

use UVTools;
use Getopt::Long;

# The water cloud table is a function of solar zenith angles and  cloudy/clear
# global irradiances ratios. The solar zenith angles and ratios in the table are
# set below. The numbers in the table are cloud optical depths for specific
# combinations of solar zenith angle and ratios.

# Solar zenith angles that will be used to calculate the wc table
@szas = (0.1, 5,10,15,
	 20,22,24,26,28,
	 30,32,34,36,38,
	 40,42,44,46,48,
	 50,51,52,53,54,55,56,57,58,59,
	 60.01,61,62,63,64,65,66,67,68,69,
	 70,71,72,73,74,75,76,77,78,79,80); 
# The cloudy/clear irradiance ratio values
# @ratio = (0.005, 0.01, 0.05, 0.075, 
# 	  0.10, 0.125, 0.15, 0.175,
# 	  0.20, 0.225, 0.25, 0.275,
# 	  0.30, 0.325, 0.35, 0.375,
# 	  0.40, 0.425, 0.45, 0.475,
# 	  0.50, 0.525, 0.55, 0.575,
# 	  0.60, 0.625, 0.65, 0.675,
# 	  0.70, 0.725, 0.75, 0.775,
# 	  0.80, 0.825, 0.85, 0.875,
# 	  0.90, 0.925, 0.95, 0.975,
# 	  1.00
# );
@ratio = (  0.0,  1.0,  2.5,  5.0,  7.5,  
           10.0,  12.5,  15.0,  17.5,  
           20.0,  22.5,  25.0,  27.5,  
           30.0,  32.5,  35.0,  37.5,  
           40.0,  42.5,  45.0,  47.5,  
           50.0,  52.5,  55.0,  57.5,  
           60.0,  62.5,  65.0,  67.5,  
           70.0,  72.5,  75.0,  77.5,  
           80.0,  82.5,  85.0,  87.5,  
           90.0,  92.5,  95.0,  97.5,  
          100.0, 125.0, 150.0, 175.0,  
          200.0, 225.0, 250.0, 275.0,  
          300.0, 325.0, 350.0, 375.0,  
          400.0, 425.0, 450.0, 475.0,  
          500.0, 525.0, 550.0, 575.0,  
          600.0, 625.0, 650.0, 675.0,  
          700.0, 725.0, 750.0, 775.0,  
          800.0, 825.0, 850.0, 875.0,  
          900.0, 925.0, 950.0, 975.0,  
         1000.0,1025.0,1050.0,1075.0,  
         1100.0,1125.0,1150.0,1175.0,  
         1200.0,1225.0,1250.0,1275.0,  
         1300.0,1325.0,1350.0,1375.0,  
         1400.0,1425.0,1450.0,1475.0,  
         1500.0,1525.0,1550.0,1575.0,  
         1600.0,1625.0,1650.0,1675.0,  
         1700.0,1725.0,1750.0,1775.0,  
         1800.0,1825.0,1850.0,1875.0,  
         1900.0,1925.0,1950.0,1975.0,  
         2000.0,2250.0,2500.0,2750.0,
         3000.0,3250.0,3500.0,3750.0,	  
         4000.0,4250.0,4500.0,4750.0,
	 5000.0
);

# $wc_lower_limit = 0.0;
# $wc_upper_limit = 500.0;
# $wc_step        = 10.0;
# $n_wc           = ($wc_upper_limit - $wc_lower_limit) / $wc_step;

# # set wc optical depth 
# for ($j=0;$j<=$n_wc;$j++) {
#     $wcs[$j] = $wc_lower_limit + $j * $wc_step;
# }

@wcs = (0.0, 0.05, 1.0, 2.0, 3.0, 4.0, 5.0, 7.5, 10.0,
        15.0, 20.0, 25.0, 30.0, 40., 50., 60., 80., 100.,
	120., 140., 160., 180., 200., 225., 250., 275.,
        300., 350., 400., 450., 500., 550., 600., 650.,
	700., 750., 800., 850., 900., 950., 1000.
	);
$n_wc = $#wcs;

$day          = 1.;    # Day number must be corrected for latter.
$ozone        = 300.0; # Ozone does not matter as we do this for wavelengths where
                       # ozone absorption does not occurr.

$UVfile = "tmp_conv";

# Default command line options 
$absolute     = 0;
$albedo       = 0.0;
$alpha        = 0.0;
$atmmod_file  = "../data/atmmod/afglus.dat";
$wc_file      = "";
$bandpassfile = "";
$beta         = 0.0;
$file         = "wc_table";
$slitfunction = "";
$lambda       = 380.0;
$o3_crs       = "Molina";

$ret = GetOptions(
		  "--absolute",\$absolute,
		  "--albedo=f",\$albedo,
		  "--alpha=f",\$alpha,
		  "--atmmod=s",\$atmmod_file,
		  "--wc_file=s",\$wc_file,
		  "--bandpass=s",\$bandpassfile,
		  "--beta=f",\$beta,
		  "--file=s",\$file,
		  "--help",\&usage,
		  "--slitfunction=s",\$slitfunction,
		  "--o3_crs=s",\$o3_crs,
		  "--lambda=f",\$lambda,
		  "<>",\&usage);
if ($ret==0) {
    exit;
}

$UVTools::uvspec_o3_crs = $o3_crs;
$UVTools::uvspec_solver = "disort";

printf STDERR "Water cloud table is written to the file $file\n";
printf STDERR "Wavelength: %f\n", $lambda;

# Now do the real things.

$bandpass = 0;
if ( length($bandpass)> 0 ) {
    $bandpass = 1;

    ($wvn1band,$wvn2band,$wvnstep) = read_bandpass($bandpassfile);
    
    $wvn1 = $wvn1band;
    $wvn2 = $wvn2band;
    if (!$absolute) {
	$wvn1 = $wvn1 + $lambda;
	$wvn2 = $wvn2 + $lambda;
    } 
    $wvncenter = $wvn1band;
    $step  = $wvnstep;
}
else {
    $wvn1 = $lambda-1.0;
    $wvn2 = $lambda+1.0;
    $wvncenter = ($wvn2+$wvn1)/2.0;
    $step  = 15.;  # Some large number to make spline only produce one number
}

$rat_file    = "tmp_rat";
$rat_wc_file = "tmp_ratwc";
$tmptable    = "tmp_table";

open(RAT,">".$rat_file);
for ($i=0;$i<=$#ratio;$i++) {
    printf RAT "%f\n", $ratio[$i];
}
close(RAT);

# First calculate the clear sky values

for ($i=0;$i<=$#szas;$i++) {

#   UVTools::Convolved_Spectrum($albedo, $day, $ozone, 
# 			      $alpha, $beta, $szas[$i], 
# 			      $wvn1, $wvn2, $wvncenter, 
# 			      $step, $atmmod_file, 
# 			      $slitfunction, $UVfile, 0.0, $wc_file);
#     if ($bandpass) {
# 	$irr_clear{$szas[$i]} = 
# 	    multiply_with_bandpass_and_integrate( $UVfile, $bandpassfile);
#     }
#     else { 
# 	$irr_clear{$szas[$i]} = read_irr($UVfile);
    $irr_clear{$szas[$i]} = 1.0;
#     }
}

# Next do all the water cloud spectra

# Loop over all zenith angles
#for ($i=0;$i<=$#szas;$i++) {
for ($i=10;$i<=10;$i++) {
    open(RAT_WC,">".$rat_wc_file);
    # Loop over all wc optical depths
#    for ($j=$n_wc;$j>=0;$j--) {
    for ($j=18;$j>=18;$j--) {

	printf STDERR "Now doing sza: %5.2f and wc: %6.3f\n", $szas[$i], $wcs[$j];

      UVTools::Convolved_Spectrum($albedo, $day, $ozone, 
				  $alpha, $beta, $szas[$i], 
				  $wvn1, $wvn2, $wvncenter, 
				  $step, $atmmod_file, 
				  $slitfunction, $UVfile, $wcs[$j], $wc_file);
	if ($bandpass) {
	    $irr = multiply_with_bandpass_and_integrate( $UVfile, $bandpassfile);
	}
	else {
	    $irr    = read_irr($UVfile);
	}
	
	$ratio_cloudy_clear = $irr/$irr_clear{$szas[$i]};
#	system("rm $UVfile");
	printf RAT_WC "%f %f\n", $ratio_cloudy_clear, $wcs[$j];
	
    }
    close(RAT_WC);
    # Interpolate calculated ratios to ratio
    system("$UVTools::spline -x $rat_file $rat_wc_file > $tmptable");
    open(TMPTABLE,$tmptable);
    @tmp = <TMPTABLE>;
    close(TMPTABLE);
    for ($k=0;$k<=$#tmp;$k++) {
	$line = @tmp[$k];
	$line =~ s/^[ ]*//;
	local($rat,$wc) = split(/[ \t]+/,$line);
	$_ = $wc;
	if ( m/[0-9\.\-\e]+/ ) {
	    $table[$k][$i] = $wc;
	}
	else {
	    $table[$k][$i] = -1;
	}
    }
}

# Write the table.  
open(TABLE,">".$file);
printf TABLE "%6.1f ", -1.0;
for ($i=0; $i<=$#szas; $i++) {
    printf TABLE "%5.0f", $szas[$i];
}
printf TABLE "\n";
  
for ($k=0;$k<=$#ratio;$k++) {
    printf TABLE "%9.5f ", $ratio[$k];
    for ($i=0; $i<=$#szas; $i++) {
	printf TABLE "%5d",  $table[$k][$i];
    }
    printf TABLE "\n";
}
close(TABLE);

sub read_irr 
{
    local($file) = shift @_;
    open(FILE,$file);
    local($line) = <FILE>;
    close(FILE);
    $line =~ s/^[ ]*//;
    local($wvn,$dir,$diff,$rest) = split(/[ \t]+/,$line);
    local($totdir) = $dir+$diff;
    $totdir;
}

sub read_bandpass
{
    local($file) = shift @_;
    open(FILE,$file) || die "read_bandpass: Could not open file: $file\n";
    local(@lines) = <FILE>;
    close(FILE);
    local($line) = @lines[0];
    $line =~ s/^[ ]*//;
    local($wvn1band,$rest) = split(/[ \t]+/,$line);
    $line = @lines[$#lines];
    $line =~ s/^[ ]*//;
    local($wvn2band,$rest) = split(/[ \t]+/,$line);
    local($wvnstep) = ($wvn2band-$wvn1band)/$#lines;
    ($wvn1band,$wvn2band, $wvnstep);
}

sub multiply_with_bandpass_and_integrate
{
    local($irrfile) = shift @_;
    local($bpfile) = shift @_;
    open(FILE,$irrfile) || die "multiply_with_bandpass_and_integrate: Could not open file: $irrfile\n";
    local(@irrlines) = <FILE>;
    close(FILE);
    open(FILE,$bpfile) || die "multiply_with_bandpass_and_integrate: Could not open file: $bpfile\n";
    local(@bplines) = <FILE>;
    close(FILE);
    local $i;

    # Multiply irradiance with bandpass
    if ($#irrlines != $#bplines)  { 
	die "multiply_with_bandpass_and_integrate: Bandpass and uvfile have different length\n";
    }
    for ($i=0;$i<=$#irrlines;$i++) {
	$line = @irrlines[$i];
#	print $line;
	chop $line;
	$line =~ s/^[ \t]+//;
	local($wvn, $dir, $diff) = split(/[ \t]+/,$line);
	$irr[$i] = $dir + $diff;
	$wvn[$i] = $wvn;
	$line = $bplines[$i];
	chop $line;
	$line =~ s/^[ \t]+//;
	local($wvnbp, $bp, $gabba) = split(/[ \t]+/,$line);
#	print "wvn  $wvn wvnbp: $wvnbp bp: $bp dir: $dir\n";
#	if ($wvn != $wvnbp)  { 
#	    die "multiply_with_bandpass_and_integrate: Bandpass and uvfile wvn differ $wvn $wvnbp\n";
#	}
	$irr[$i] = $irr[$i]*$bp;
    }

    # Integrate up the irradiance

    $tmpbp = "tmpbp";
    $tmpirr = "tmpirr";
    open(BP,">".$tmpbp);
    for ($i=0;$i<=$#irrlines;$i++) {
	printf BP "%f %f\n", $wvn[$i], $irr[$i];
    }
    close(BP);

    system("$UVTools::integrate -p  $tmpbp > $tmpirr");

    open(IRR,$tmpirr);
    $irr = <IRR>;
    chop $irr;
    close(IRR);

#    system("$UVTools::conv $irrfile $bpfile > tmpconv");

#    die "multi $irr die\n";
    $irr;

}

sub usage {
    printf STDERR "\n";
    printf STDERR "Gen_wc_tab generates a table of cloudy/clear sky irradiance\n";
    printf STDERR "ratios versus solar zenith angle for different water cloud\n";
    printf STDERR "optical depth for a given wavelength. The table is\n";
    printf STDERR "read and water cloud optical depth derived by the read_o3_tab program.\n";
    printf STDERR "\n";
    printf STDERR "The irradiance ratios may be calculated in the following ways:\n";
    printf STDERR "(See the libRadtran documentation for examples.)\n";
    printf STDERR "\n";
    printf STDERR "   * Cloudy/clear ratio calculated from single wavelength\n";
    printf STDERR "     measurements.\n";
    printf STDERR "   * Cloudy/clear ratio calculated from measurements multiplied by\n";
    printf STDERR "     a bandpass function and integrated over the bandpass.\n";
    printf STDERR "\n";
    printf STDERR "Gen_wc_tab understands the following options:\n";
    printf STDERR "\n";
    printf STDERR "--absolute               : The wavelengths of the bandpass file are in\n";
    printf STDERR "                           absolute units. Default is relative units.\n";
    printf STDERR "--albedo <value>         : Lambertian surface albedo. Default is 0.0.\n";
    printf STDERR "--alpha <value>          : Angstrom alpha coefficient. Default is 0.0.\n";
    printf STDERR "--atmmod <name>          : Name of atmosphere file. Default atmmod/afglus.dat.\n";
    printf STDERR "--wc_file <name>         : Name of water cloud file. Default none.\n";
    printf STDERR "--beta <value>           : Angstrom beta coefficient. Default is 0.0.\n";
    printf STDERR "--help                   : Prints this message.\n";
    printf STDERR "--o3_crs <name>          : Name of o3 cross section to use. Default is\n";
    printf STDERR "                           Molina. See libRadtran documentation for other options.\n";
    printf STDERR "--slitfunction <name>    : Name of slitfunction file.\n";
    printf STDERR "--bandpass <name>        : Name of file holding bandpass for wavelength.\n";
    printf STDERR "--file <name>            : Name of file where the table will be stored.\n";
    printf STDERR "--lambda <value>         : Value for wavelength, in nm.\n";
    printf STDERR "\n";
    exit;
}

# Local Variables:
# mode: Perl
# End:
